#define atmosphereHeight 8228.0
#define earthRadius 6371000.0
#define ozoneMultiplier 0.01
#define rayleighDistribution 8.0
#define mieDistribution 1.8
#define mieCoefficient 0.000003

#define rayleighCoefficient vec3(0.0000031965, 0.00000976, 0.00003013)
#define ozoneCoefficient vec3(0.00020556, 0.00049788, 0.00002136)

#define phaseRayleigh(a) (0.4 * a + 1.12)
#define divFix(a) (abs(a) + 1e-8)

c(vec3) skyCoeff = (ozoneCoefficient * ozoneMultiplier + rayleighCoefficient) * rcpln2;

c(vec2) radius = earthRadius + vec2(atmosphereHeight, atmosphereHeight * mieDistribution / rayleighDistribution);

vec3 radiusDirection = -upDirection * earthRadius;

float radiusDepth = dotX(radiusDirection);

float saturationLevel = mix(1.0, 0.3, rainStrength);
float nightDifference = mix(1.0, 0.25, timeNight);

vec2 atmosphericThickness(in vec3 lightPosition) {

    float b = dot(lightPosition, radiusDirection);

    float t = b * b - radiusDepth;
	
    return b + sqrt(radius * radius + t);
}

float phaseMie(float x, float g) {
    return ((g * g) * -0.25 + 0.25) * exp2(-(log2(-2.0 * (g * x) + (g * g + 1.0)) * 1.5));
}

mat2x2 lightThickness = mat2x2(atmosphericThickness(lightPosition[0]), atmosphericThickness(lightPosition[1]) * nightDifference);
mat2x3 lightCoeff = mat2x3(lightThickness[0].x * skyCoeff + (lightThickness[0].y * mieCoefficient), lightThickness[1].x * skyCoeff + (lightThickness[1].y * mieCoefficient));
mat2x3 lightAbsorb = mat2x3(exp2(-lightCoeff[0]), exp2(-lightCoeff[1]));

void computeSkyColor(io vec3 skyOutput, in vec3 viewSpacePosition) {

	float VdotL = dot(viewSpacePosition, lightPosition[0]);
	
	vec2 thickness = atmosphericThickness(viewSpacePosition);

	vec3 viewCoeff = thickness.x * skyCoeff + (thickness.y * mieCoefficient);

	vec3 absorption = divFix(lightAbsorb[0] - exp2(-viewCoeff)) / divFix((lightCoeff[0] - viewCoeff) * ln2);

	vec4 scatter = vec4(0.0);

	scatter.xyz = thickness.x * phaseRayleigh(VdotL) * rayleighCoefficient;

	scatter.w = thickness.y * phaseMie(VdotL, exp2(thickness.y * -0.0002)) * mieCoefficient;

	skyOutput = (scatter.xyz + scatter.w) * absorption;

	skyOutput = saturate(skyOutput, clamp01(saturationLevel - timeNight), luma(skyOutput)) * saturationLevel;
	
	skyOutput = clamp01(skyOutput);
}

vec3 computeSkyRayleigh(in vec3 viewSpacePosition) {

	vec2 thickness = atmosphericThickness(viewSpacePosition);
	
	vec3 viewCoeff = thickness.x * skyCoeff + (thickness.y * mieCoefficient);

	vec3 viewAbsorb = exp2(-viewCoeff);

	vec3 absorb = divFix(lightAbsorb[0] - viewAbsorb) / divFix((lightCoeff[0] - viewCoeff) * ln2);
	
	return (thickness.x * phaseRayleigh(dot(viewSpacePosition, lightPosition[0])) * rayleighCoefficient) * absorb;
}

vec3 computeTones(io vec3 absorb, io vec3 rayleigh) {

	vec2 thickness = sqrt(radius * radius - radiusDepth);
	
	vec3 viewCoeff = thickness.x * skyCoeff + (thickness.y * mieCoefficient);

	rayleigh = (thickness.x * rayleighCoefficient);

	absorb = divFix(lightAbsorb[1] - exp2(-viewCoeff)) / divFix((lightCoeff[1] - viewCoeff) * ln2);

	vec3 tone = rayleigh * absorb;
	
	rayleigh = fPow(tone, (mieDistribution - inversesqrt(rayleighCoefficient / mieCoefficient)) * 0.75) / vec3(1.0, 0.899, 0.828);

	rayleigh = saturate(rayleigh, clamp01(saturationLevel - timeNight * 0.5), luma(rayleigh));
	
	absorb = mix(absorb, 1.0 - absorb, timeNight);

	return tone * inversesqrt(dotX(tone) + 1.0);

}